@@ -1,5 +1,5 @@
 class Reply < ActiveRecord::Base
-  named_scope :base
+  scope :base
   belongs_to :topic, :include => [:replies]
   belongs_to :developer
 
