@@ -1,4 +1,5 @@
 require 'abstract_unit'
+require 'active_support/core_ext/module'
 
 module One
   Constant1 = "Hello World"
@@ -25,15 +26,20 @@ module Yz
   end
 end
 
-class De
+Somewhere = Struct.new(:street, :city) do
+  attr_accessor :name
 end
 
-Somewhere = Struct.new(:street, :city)
-
-Someone   = Struct.new(:name, :place) do
-  delegate :street, :city, :to => :place
-  delegate :state, :to => :@place
+class Someone < Struct.new(:name, :place)
+  delegate :street, :city, :to_f, :to => :place
+  delegate :name=, :to => :place, :prefix => true
   delegate :upcase, :to => "place.city"
+
+  FAILED_DELEGATE_LINE = __LINE__ + 1
+  delegate :foo, :to => :place
+
+  FAILED_DELEGATE_LINE_2 = __LINE__ + 1
+  delegate :bar, :to => :place, :allow_nil => true
 end
 
 Invoice   = Struct.new(:client) do
@@ -41,6 +47,19 @@ Invoice   = Struct.new(:client) do
   delegate :street, :city, :name, :to => :client, :prefix => :customer
 end
 
+Project   = Struct.new(:description, :person) do
+  delegate :name, :to => :person, :allow_nil => true
+  delegate :to_f, :to => :description, :allow_nil => true
+end
+
+Developer = Struct.new(:client) do
+  delegate :name, :to => :client, :prefix => nil
+end
+
+Tester = Struct.new(:client) do
+  delegate :name, :to => :client, :prefix => false
+end
+
 class Name
   delegate :upcase, :to => :@full_name
 
@@ -49,35 +68,21 @@ class Name
   end
 end
 
-$nowhere = <<-EOF
-class Name
-  delegate :nowhere
-end
-EOF
-
-$noplace = <<-EOF
-class Name
-  delegate :noplace, :tos => :hollywood
-end
-EOF
-
 class ModuleTest < Test::Unit::TestCase
   def setup
     @david = Someone.new("David", Somewhere.new("Paulina", "Chicago"))
   end
 
-  def test_included_in_classes
-    assert One.included_in_classes.include?(Ab)
-    assert One.included_in_classes.include?(Xy::Bc)
-    assert One.included_in_classes.include?(Yz::Zy::Cd)
-    assert !One.included_in_classes.include?(De)
-  end
-
   def test_delegation_to_methods
     assert_equal "Paulina", @david.street
     assert_equal "Chicago", @david.city
   end
 
+  def test_delegation_to_assignment_method
+    @david.place_name = "Fred"
+    assert_equal "Fred", @david.place.name
+  end
+
   def test_delegation_down_hierarchy
     assert_equal "CHICAGO", @david.upcase
   end
@@ -88,8 +93,12 @@ class ModuleTest < Test::Unit::TestCase
   end
 
   def test_missing_delegation_target
-    assert_raises(ArgumentError) { eval($nowhere) }
-    assert_raises(ArgumentError) { eval($noplace) }
+    assert_raise(ArgumentError) do
+      Name.send :delegate, :nowhere
+    end
+    assert_raise(ArgumentError) do
+      Name.send :delegate, :noplace, :tos => :hollywood
+    end
   end
 
   def test_delegation_prefix
@@ -106,6 +115,11 @@ class ModuleTest < Test::Unit::TestCase
     assert_equal invoice.customer_city, "Chicago"
   end
 
+  def test_delegation_prefix_with_nil_or_false
+    assert_equal Developer.new(@david).name, "David"
+    assert_equal Tester.new(@david).name, "David"
+  end
+
   def test_delegation_prefix_with_instance_variable
     assert_raise ArgumentError do
       Class.new do
@@ -117,6 +131,73 @@ class ModuleTest < Test::Unit::TestCase
     end
   end
 
+  def test_delegation_with_allow_nil
+    rails = Project.new("Rails", Someone.new("David"))
+    assert_equal rails.name, "David"
+  end
+
+  def test_delegation_with_allow_nil_and_nil_value
+    rails = Project.new("Rails")
+    assert_nil rails.name
+  end
+
+  def test_delegation_with_allow_nil_and_nil_value_and_prefix
+    Project.class_eval do
+      delegate :name, :to => :person, :allow_nil => true, :prefix => true
+    end
+    rails = Project.new("Rails")
+    assert_nil rails.person_name
+  end
+
+  def test_delegation_without_allow_nil_and_nil_value
+    david = Someone.new("David")
+    assert_raise(RuntimeError) { david.street }
+  end
+
+  def test_delegation_to_method_that_exists_on_nil
+    nil_person = Someone.new(nil)
+    assert_equal 0.0, nil_person.to_f
+  end
+
+  def test_delegation_to_method_that_exists_on_nil_when_allowing_nil
+    nil_project = Project.new(nil)
+    assert_equal 0.0, nil_project.to_f
+  end
+
+  def test_delegation_does_not_raise_error_when_removing_singleton_instance_methods
+    parent = Class.new do
+      def self.parent_method; end
+    end
+
+    assert_nothing_raised do
+      Class.new(parent) do
+        class << self
+          delegate :parent_method, :to => :superclass
+        end
+      end
+    end
+  end
+
+  def test_delegation_exception_backtrace
+    someone = Someone.new("foo", "bar")
+    someone.foo
+  rescue NoMethodError => e
+    file_and_line = "#{__FILE__}:#{Someone::FAILED_DELEGATE_LINE}"
+    # We can't simply check the first line of the backtrace, because JRuby reports the call to __send__ in the backtrace.
+    assert e.backtrace.any?{|a| a.include?(file_and_line)},
+           "[#{e.backtrace.inspect}] did not include [#{file_and_line}]"
+  end
+
+  def test_delegation_exception_backtrace_with_allow_nil
+    someone = Someone.new("foo", "bar")
+    someone.bar
+  rescue NoMethodError => e
+    file_and_line = "#{__FILE__}:#{Someone::FAILED_DELEGATE_LINE_2}"
+    # We can't simply check the first line of the backtrace, because JRuby reports the call to __send__ in the backtrace.
+    assert e.backtrace.any?{|a| a.include?(file_and_line)},
+           "[#{e.backtrace.inspect}] did not include [#{file_and_line}]"
+  end
+
   def test_parent
     assert_equal Yz::Zy, Yz::Zy::Cd.parent
     assert_equal Yz, Yz::Zy.parent
@@ -131,11 +212,6 @@ class ModuleTest < Test::Unit::TestCase
   def test_local_constants
     assert_equal %w(Constant1 Constant3), Ab.local_constants.sort.map(&:to_s)
   end
-
-  def test_as_load_path
-    assert_equal 'yz/zy', Yz::Zy.as_load_path
-    assert_equal 'yz', Yz.as_load_path
-  end
 end
 
 module BarMethodAliaser
@@ -192,7 +268,7 @@ class MethodAliasingTest < Test::Unit::TestCase
     FooClassWithBarMethod.class_eval { include BarMethodAliaser }
 
     feature_aliases.each do |method|
-      assert @instance.respond_to?(method)
+      assert_respond_to @instance, method
     end
 
     assert_equal 'bar_with_baz', @instance.bar
@@ -209,7 +285,7 @@ class MethodAliasingTest < Test::Unit::TestCase
       include BarMethodAliaser
       alias_method_chain :quux!, :baz
     end
-    assert @instance.respond_to?(:quux_with_baz!)
+    assert_respond_to @instance, :quux_with_baz!
 
     assert_equal 'quux_with_baz', @instance.quux!
     assert_equal 'quux', @instance.quux_without_baz!
@@ -227,9 +303,9 @@ class MethodAliasingTest < Test::Unit::TestCase
     assert !@instance.respond_to?(:quux_with_baz=)
 
     FooClassWithBarMethod.class_eval { include BarMethodAliaser }
-    assert @instance.respond_to?(:quux_with_baz!)
-    assert @instance.respond_to?(:quux_with_baz?)
-    assert @instance.respond_to?(:quux_with_baz=)
+    assert_respond_to @instance, :quux_with_baz!
+    assert_respond_to @instance, :quux_with_baz?
+    assert_respond_to @instance, :quux_with_baz=
 
 
     FooClassWithBarMethod.alias_method_chain :quux!, :baz
@@ -287,7 +363,7 @@ class MethodAliasingTest < Test::Unit::TestCase
       alias_method_chain :duck, :orange
     end
 
-    assert_raises NoMethodError do
+    assert_raise NoMethodError do
       @instance.duck
     end
 
@@ -303,7 +379,7 @@ class MethodAliasingTest < Test::Unit::TestCase
       alias_method_chain :duck, :orange
     end
 
-    assert_raises NoMethodError do
+    assert_raise NoMethodError do
       @instance.duck
     end
 
